@@ -7,7 +7,9 @@ G_BEGIN_DECLS
 
 typedef enum {
     PREF_GENERAL,
-    PREF_DESKTOP
+    PREF_DESKTOP,
+    PREF_VOLMAN,
+    PREF_ADVANCED
 }PrefDlgPage;
 
 gboolean fm_edit_preference( GtkWindow* parent, int page );
