@@ -102,6 +102,7 @@ gboolean open_up_progress_dlg( PtkFileTask* task )
         {
             N_( "Move: " ),
             N_( "Copy: " ),
+            N_( "Move to Trash: " ),
             N_( "Delete: " ),
             N_( "Link: " ),
             N_( "Change Properties: " )
@@ -110,6 +111,7 @@ gboolean open_up_progress_dlg( PtkFileTask* task )
         {
             N_( "Moving..." ),
             N_( "Copying..." ),
+            N_( "Moving to Trash..." ),
             N_( "Deleting..." ),
             N_( "Linking..." ),
             N_( "Changing Properties" )
