@@ -12,6 +12,15 @@
 
 #include "ptk-path-entry.h"
 #include <gdk/gdkkeysyms.h>
+#include "vfs-file-info.h"  /* for vfs_file_resolve_path */
+#include <string.h>
+
+enum
+{
+    COL_NAME,
+    COL_PATH,
+    N_COLS
+};
 
 /*
 static GQuark use_hand_cursor = (GQuark)"hand_cursor";
@@ -21,21 +30,57 @@ static GQuark use_hand_cursor = (GQuark)"hand_cursor";
 static char*
 get_cwd( GtkEntry* entry )
 {
-    if( g_path_is_absolute( gtk_entry_get_text(entry) ) )
-        return g_path_get_dirname( gtk_entry_get_text(entry) );
+    char *real_path = vfs_file_resolve_path( NULL, gtk_entry_get_text(entry) );
+    if( G_LIKELY( g_path_is_absolute( real_path ) ) )
+    {
+        char* ret = g_path_get_dirname( real_path );
+        g_free( real_path );
+        return ret;
+    }
     return NULL;
 }
 
+static gboolean match_func( GtkEntryCompletion *completion,
+                                                     const gchar *key,
+                                                     GtkTreeIter *it,
+                                                     gpointer user_data)
+{
+    char* name = NULL;
+    GtkTreeModel* model = gtk_entry_completion_get_model(completion);
+
+    key = (const char*)g_object_get_data( G_OBJECT(completion), "fn" );
+    gtk_tree_model_get( model, it, COL_NAME, &name, -1 );
+
+    if( G_LIKELY(name) )
+    {
+        if( *key == 0 || 0 == g_ascii_strncasecmp( name, key, strlen(key) ) )
+        {
+            g_free( name );
+            return TRUE;
+        }
+        g_free( name );
+    }
+    return FALSE;
+}
+
 static void update_completion( GtkEntry* entry,
                                GtkEntryCompletion* completion )
 {
-    char* new_dir;
+    char* new_dir, *fn;
     const char* old_dir;
     GtkListStore* list;
+    const char *sep;
+
+    sep = strrchr( gtk_entry_get_text(entry), '/' );
+    if( sep )
+        fn = (char*)sep + 1;
+    else
+        fn = (char*)gtk_entry_get_text(entry);
+    g_object_set_data_full( G_OBJECT(completion), "fn", g_strdup(fn), (GDestroyNotify)g_free );
 
     new_dir = get_cwd( entry );
     old_dir = (const char*)g_object_get_data( (GObject*)completion, "cwd" );
-    if( old_dir && new_dir && 0 == strcmp( old_dir, new_dir ) )
+    if( old_dir && new_dir && 0 == g_ascii_strcasecmp( old_dir, new_dir ) )
     {
         g_free( new_dir );
         return;
@@ -47,24 +92,28 @@ static void update_completion( GtkEntry* entry,
     if( new_dir )
     {
         GDir* dir;
-        if( dir = g_dir_open( new_dir, 0, NULL ) )
+        if( (dir = g_dir_open( new_dir, 0, NULL )) )
         {
             const char* name;
-            while( name = g_dir_read_name( dir ) )
+            while( (name = g_dir_read_name( dir )) )
             {
                 char* full_path = g_build_filename( new_dir, name, NULL );
                 if( g_file_test( full_path, G_FILE_TEST_IS_DIR ) )
                 {
                     GtkTreeIter it;
-                    char* disp_path = g_filename_display_name( full_path );
+                    char* disp_name = g_filename_display_basename( full_path );
                     gtk_list_store_append( list, &it );
-                    gtk_list_store_set( list, &it, 0, disp_path, -1 );
-                    g_free( disp_path );
+                    gtk_list_store_set( list, &it, COL_NAME, disp_name, COL_PATH, full_path, -1 );
+                    g_free( disp_name );
                 }
                 g_free( full_path );
             }
             g_dir_close( dir );
+
+            gtk_entry_completion_set_match_func( completion, match_func, new_dir, NULL );
         }
+        else
+            gtk_entry_completion_set_match_func( completion, NULL, NULL, NULL );
     }
 }
 
@@ -86,7 +135,7 @@ on_key_press( GtkWidget *entry, GdkEventKey* evt, gpointer user_data )
            related to this API which cause crash.
            Reported in bug #1570063 by Orlando Fiol <fiolorlando@gmail.com>
         */
-        gtk_entry_completion_insert_prefix( gtk_entry_get_completion(entry) );
+        gtk_entry_completion_insert_prefix( gtk_entry_get_completion(GTK_ENTRY(entry)) );
 #endif
         gtk_editable_set_position( (GtkEditable*)entry, -1 );
         return TRUE;
@@ -98,18 +147,26 @@ static gboolean
 on_focus_in( GtkWidget *entry, GdkEventFocus* evt, gpointer user_data )
 {
     GtkEntryCompletion* completion = gtk_entry_completion_new();
-    GtkListStore* list = gtk_list_store_new( 1, G_TYPE_STRING );
+    GtkListStore* list = gtk_list_store_new( N_COLS, G_TYPE_STRING, G_TYPE_STRING );
+    GtkCellRenderer* render;
+
     gtk_entry_completion_set_minimum_key_length( completion, 1 );
     gtk_entry_completion_set_model( completion, GTK_TREE_MODEL(list) );
     g_object_unref( list );
-    gtk_entry_completion_set_text_column( completion, 0 );
+
+    /* gtk_entry_completion_set_text_column( completion, COL_PATH ); */
+    g_object_set( completion, "text-column", COL_PATH, NULL );
+    render = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start( (GtkCellLayout*)completion, render, TRUE );
+    gtk_cell_layout_add_attribute( (GtkCellLayout*)completion, render, "text", COL_NAME );
+
     gtk_entry_completion_set_inline_completion( completion, TRUE );
 #if GTK_CHECK_VERSION( 2, 8, 0)
     /* gtk+ prior to 2.8.0 doesn't have this API */
     gtk_entry_completion_set_popup_set_width( completion, TRUE );
 #endif
     gtk_entry_set_completion( GTK_ENTRY(entry), completion );
-    g_signal_connect( entry, "changed", on_changed, NULL );
+    g_signal_connect( G_OBJECT(entry), "changed", G_CALLBACK(on_changed), NULL );
     g_object_unref( completion );
 
     return FALSE;
@@ -118,8 +175,8 @@ on_focus_in( GtkWidget *entry, GdkEventFocus* evt, gpointer user_data )
 static gboolean
 on_focus_out( GtkWidget *entry, GdkEventFocus* evt, gpointer user_data )
 {
-    gtk_entry_set_completion( GTK_ENTRY(entry), NULL );
     g_signal_handlers_disconnect_by_func( entry, on_changed, NULL );
+    gtk_entry_set_completion( GTK_ENTRY(entry), NULL );
     return FALSE;
 }
 
