@@ -50,7 +50,7 @@ const char* vfs_app_desktop_get_disp_name( VFSAppDesktop* app );
 
 const char* vfs_app_desktop_get_exec( VFSAppDesktop* app );
 
-GdkPixbuf* vfs_app_desktop_get_icon( VFSAppDesktop* app, int size );
+GdkPixbuf* vfs_app_desktop_get_icon( VFSAppDesktop* app, int size, gboolean use_fallback );
 
 const char* vfs_app_desktop_get_icon_name( VFSAppDesktop* app );
 
