@@ -20,6 +20,7 @@
 
 #include "vfs-app-desktop.h"
 #include "vfs-thumbnail-loader.h"
+#include "vfs-utils.h"  /* for vfs_load_icon */
 
 static int big_thumb_size = 48, small_thumb_size = 20;
 static gboolean utf8_file_name = FALSE;
@@ -218,6 +219,7 @@ void vfs_file_info_reload_mime_type( VFSFileInfo* fi,
     old_mime_type = fi->mime_type;
     fi->mime_type = vfs_mime_type_get_from_file( full_path,
                                                  fi->name, &file_stat );
+    vfs_file_info_load_special_info( fi, full_path );
     vfs_mime_type_unref( old_mime_type );  /* FIXME: is vfs_mime_type_unref needed ?*/
 }
 
@@ -259,10 +261,9 @@ GdkPixbuf* vfs_file_info_get_big_icon( VFSFileInfo* fi )
                 if ( G_UNLIKELY( icon_name[ 0 ] == '/' ) )
                     fi->big_thumbnail = gdk_pixbuf_new_from_file( icon_name, NULL );
                 else
-                    fi->big_thumbnail = gtk_icon_theme_load_icon(
+                    fi->big_thumbnail = vfs_load_icon(
                                             gtk_icon_theme_get_default(),
-                                            icon_name,
-                                            icon_size, 0, NULL );
+                                            icon_name, icon_size );
             }
             if ( fi->big_thumbnail )
                 g_object_set_data_full( G_OBJECT(fi->big_thumbnail), "name", icon_name, g_free );
@@ -459,6 +460,11 @@ gboolean vfs_file_info_is_image( VFSFileInfo* fi )
     return FALSE;
 }
 
+gboolean vfs_file_info_is_desktop_entry( VFSFileInfo* fi )
+{
+    return 0 != (fi->flags & VFS_FILE_INFO_DESKTOP_ENTRY);
+}
+
 gboolean vfs_file_info_is_unknown_type( VFSFileInfo* fi )
 {
     if ( ! strcmp( XDG_MIME_TYPE_UNKNOWN,
@@ -496,7 +502,7 @@ gboolean vfs_file_info_open_file( VFSFileInfo* fi,
 
     if ( vfs_file_info_is_executable( fi, file_path ) )
     {
-        argv[ 0 ] = file_path;
+        argv[ 0 ] = (char *) file_path;
         argv[ 1 ] = '\0';
         ret = g_spawn_async( NULL, argv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL|
                              G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, err );
@@ -510,7 +516,7 @@ gboolean vfs_file_info_open_file( VFSFileInfo* fi,
             app = vfs_app_desktop_new( app_name );
             if ( ! vfs_app_desktop_get_exec( app ) )
                 app->exec = g_strdup( app_name );   /* FIXME: app->exec */
-            files = g_list_prepend( files, file_path );
+            files = g_list_prepend( files, (gpointer) file_path );
             /* FIXME: working dir is needed */
             ret = vfs_app_desktop_open_files( gdk_screen_get_default(),
                                               NULL, app, files, err );
@@ -579,10 +585,12 @@ void vfs_file_info_set_thumbnail_size( int big, int small )
 void vfs_file_info_load_special_info( VFSFileInfo* fi,
                                       const char* file_path )
 {
-    if ( g_str_has_suffix( fi->name, ".desktop" ) )
+    /*if ( G_LIKELY(fi->type) && G_UNLIKELY(fi->type->name, "application/x-desktop") ) */
+    if ( G_UNLIKELY( g_str_has_suffix( fi->name, ".desktop") ) )
     {
         VFSAppDesktop * desktop;
         const char* icon_name;
+
         fi->flags |= VFS_FILE_INFO_DESKTOP_ENTRY;
         desktop = vfs_app_desktop_new( file_path );
         if ( vfs_app_desktop_get_disp_name( desktop ) )
@@ -590,38 +598,24 @@ void vfs_file_info_load_special_info( VFSFileInfo* fi,
             vfs_file_info_set_disp_name(
                 fi, vfs_app_desktop_get_disp_name( desktop ) );
         }
-        if( fi->big_thumbnail )
-        {
-            gdk_pixbuf_unref( fi->big_thumbnail );
-            fi->big_thumbnail = NULL;
-        }
+
         if ( (icon_name = vfs_app_desktop_get_icon_name( desktop )) )
         {
-            char * _icon_name = strdup( icon_name );
             GdkPixbuf* icon;
-            int size;
-            vfs_mime_type_get_icon_size( &size, NULL );
-            /* icon name is a full path */
-            if ( icon_name[ 0 ] == '/' )
+            int big_size, small_size;
+            vfs_mime_type_get_icon_size( &big_size, &small_size );
+            if( ! fi->big_thumbnail )
             {
-                icon = gdk_pixbuf_new_from_file_at_size(
-                           icon_name, size, size, NULL );
+                icon = vfs_app_desktop_get_icon( desktop, big_size, FALSE );
+                if( G_LIKELY(icon) )
+                    fi->big_thumbnail =icon;
             }
-            else
+            if( ! fi->small_thumbnail )
             {
-                char* dot = strchr( _icon_name, '.' );
-                if ( dot )
-                    * dot = '\0';
-                icon = gtk_icon_theme_load_icon(
-                           gtk_icon_theme_get_default(),
-                           _icon_name, size, 0, NULL );
+                icon = vfs_app_desktop_get_icon( desktop, small_size, FALSE );
+                if( G_LIKELY(icon) )
+                    fi->small_thumbnail =icon;
             }
-            /* save app icon in thumbnail */
-            fi->big_thumbnail = icon;
-            if ( G_LIKELY( icon ) )
-                g_object_set_data_full( G_OBJECT(icon), "name", _icon_name, g_free );
-            else
-                g_free( _icon_name );
         }
         vfs_app_desktop_unref( desktop );
     }
@@ -637,7 +631,14 @@ void vfs_file_info_list_free( GList* list )
 char* vfs_file_resolve_path( const char* cwd, const char* relative_path )
 {
     GString* ret = g_string_sized_new( 4096 );
-    const char* sep;
+    int len;
+    gboolean strip_tail;
+
+    g_return_val_if_fail( G_LIKELY(relative_path), NULL );
+
+    len = strlen( relative_path );
+    strip_tail = (0 == len || relative_path[len-1] != '/');
+
     if( G_UNLIKELY(*relative_path != '/') ) /* relative path */
     {
         if( G_UNLIKELY(relative_path[0] == '~') ) /* home dir */
@@ -649,16 +650,17 @@ char* vfs_file_resolve_path( const char* cwd, const char* relative_path )
         {
             if( ! cwd )
             {
-                cwd = g_get_current_dir();
-                g_string_append( ret, cwd );
-                g_free( cwd );
+                char *cwd_new;
+                cwd_new = g_get_current_dir();
+                g_string_append( ret, cwd_new );
+                g_free( cwd_new );
             }
             else
                 g_string_append( ret, cwd );
         }
     }
 
-    if( relative_path[0] != '/' )
+    if( relative_path[0] != '/'  && (0 == ret->len || ret->str[ ret->len - 1 ] != '/' ) )
         g_string_append_c( ret, '/' );
 
     while( G_LIKELY( *relative_path ) )
@@ -688,8 +690,8 @@ char* vfs_file_resolve_path( const char* cwd, const char* relative_path )
         }while( G_LIKELY( *(relative_path++) != '/' && *relative_path ) );
     }
 
-    /* remove tailing '/' */
-    if( G_LIKELY( ret->len > 1 ) && G_UNLIKELY( ret->str[ ret->len - 1 ] == '/' ) )
+    /* if original path contains tailing '/', preserve it; otherwise, remove it. */
+    if( strip_tail && G_LIKELY( ret->len > 1 ) && G_UNLIKELY( ret->str[ ret->len - 1 ] == '/' ) )
         g_string_truncate( ret, ret->len - 1 );
     return g_string_free( ret, FALSE );
 }
